@@ -235,6 +235,7 @@ create.dmdf=function(x,parameter,time.varying=NULL,fields=NULL)
 	   begin.time=x$data$begin.time
 #  create base df
    df=create.base.dmdf(x,parameter)
+   if(is.null(df))return(NULL)
 #  add time,age,choort
    if(length(begin.time)==1 & is.vector(time.intervals))
    {
@@ -270,13 +271,13 @@ create.dmdf=function(x,parameter,time.varying=NULL,fields=NULL)
 #  if there are any time varying covariates then construct the data matrix for
 #  those covariates. That matrix is appended to other data in x that is non-time varying
    times=df$time[df$id==1]
-   occ=1:(nocc+parameter$num)
+ #  occ=1:(nocc+parameter$num)
    tcv=NULL
    if(!is.null(time.varying))
    {
 	   for (i in 1:length(time.varying))
 	   {
-		   vnames=paste(time.varying[i],times[occ],sep="")     
+		   vnames=paste(time.varying[i],times,sep="")     
 		   if( !all(vnames %in% names(x$data)))
 			   stop("Missing time varying variable ",paste(vnames[!vnames%in%names(x$data)],collapse=","))
 		   if(i==1) 
@@ -299,11 +300,12 @@ create.dmdf=function(x,parameter,time.varying=NULL,fields=NULL)
    df$seq=NULL
    df$Time=df$time-min(df$time)
    df$Cohort=df$cohort-min(df$cohort)
-   df$Age=df$age-max(0,min(df$age))
+  # df$Age=df$age-max(0,min(df$age))
    df$age[df$age<0]=0
-   df$time=factor(df$time) 
+   df$time=factor(df$time)
+   df$Age=df$age
    df$age=factor(df$age) 
-   df$cohort=factor(df$cohort) 
+   df$cohort=factor(df$cohort)
    if("group"%in%names(df))
 	  levels(df$group)=apply(x$group.covariates,1,paste,collapse="")
    if(!is.null(x$strata_data)&!is.null(df$stratum))
@@ -319,75 +321,103 @@ create.base.dmdf=function(x,parameter)
 	occasions=begin.num:(parameter$begin+nocc)
 	sl=factor(x$strata.labels)
 	nstrata=length(sl)
-	if(!is.null(x$strata.list))
-	{
-		st.index=which("states"==names(x$strata.list))
-		oth.index=which("states"!=names(x$strata.list))
-		oth.name=names(x$strata.list)[oth.index]
-		oth=factor(rep(x$strata.list[[oth.index]],each=length(x$strata.list$states)))
-		states=factor(rep(x$strata.list$states,times=length(x$strata.list[[oth.index]])))
-		if(parameter$whichlevel!=0)
-		{
-			if(parameter$whichlevel==1)
-				sl=factor(x$strata.list$states)
-			else
-				sl=factor(x$strata.list[[oth.index]])
-			nstrata=length(sl)
-			x$strata.list=NULL
-		}
-	}
-	if(!parameter$bystratum)
+    # MVMS model is currently split off but eventually this should be merged in with
+	# other bi-level MS models
+	if(nchar(x$model)>=4 & substr(x$model,1,4)=="MVMS")
 	{
+		if(!is.null(parameter$obs) & parameter$obs)
+			if(any(x$strata.list$uncertain))
+			   dfl=mvms_design_data(x$strata.list$df.states,x$strata.list$df,transition=parameter$tostrata)
+	        else
+				return(NULL)
+		else
+		   dfl=mvms_design_data(x$strata.list$df.states,transition=parameter$tostrata)
 		df=expand.grid(occ=occasions,id=factor(1:nrow(x$data)))
-		df$seq=1:nrow(df)
-	}
-	else
-	{
+		dfl=dfl[rep(1:nrow(dfl),each=nrow(df)),,drop=FALSE]
+		df=cbind(df,dfl)
 		if(parameter$tostrata)
+			df=df[order(df$id,df$occ,df$stratum,df$tostratum),]
+		else
+			df=df[order(df$id,df$occ,df$stratum),]
+		df$seq=1:nrow(df)	
+		rownames(df)=df$seq
+	} else
+	{
+		if(!is.null(x$strata.list))
 		{
-			df=expand.grid(tostratum=sl[1:nstrata],stratum=sl[1:nstrata],occ=occasions,id=factor(1:nrow(x$data)))
-			df=df[,c("stratum","tostratum","occ","id")]
+			st.index=which("states"==names(x$strata.list))
+			oth.index=which("states"!=names(x$strata.list))
+			oth.name=names(x$strata.list)[oth.index]
+			oth=factor(rep(x$strata.list[[oth.index]],each=length(x$strata.list$states)))
+			states=factor(rep(x$strata.list$states,times=length(x$strata.list[[oth.index]])))
+			# For 2-level independent strata model 
+			if(parameter$whichlevel!=0)
+			{
+				if(parameter$whichlevel==1)
+					sl=factor(x$strata.list$states)
+				else
+					sl=factor(x$strata.list[[oth.index]])
+				nstrata=length(sl)
+				x$strata.list=NULL
+			}
+		}
+		# not by stratum
+		if(!parameter$bystratum)
+		{
+			df=expand.grid(occ=occasions,id=factor(1:nrow(x$data)))
 			df$seq=1:nrow(df)
 		}
 		else
 		{
-			df=expand.grid(stratum=sl[1:nstrata],occ=occasions,id=factor(1:nrow(x$data)))
-			df$seq=1:nrow(df)
-		}
-	    if(!is.null(x$strata.list))
-	    {
-            state.df=data.frame(stratum=sl[1:nstrata],state=states,oth=oth)	
-			df=merge(df,state.df,by="stratum")
+			# from stratum - to stratum transition
 			if(parameter$tostrata)
 			{
-				state.df=data.frame(tostratum=sl[1:nstrata],tostate=states,tooth=oth)	
-				df=merge(df,state.df,by="tostratum")
-			} 
-			if(parameter$whichlevel!=0)
+				df=expand.grid(tostratum=sl[1:nstrata],stratum=sl[1:nstrata],occ=occasions,id=factor(1:nrow(x$data)))
+				df=df[,c("stratum","tostratum","occ","id")]
+				df$seq=1:nrow(df)
+			}
+			# by stratum
+			else
 			{
-				if(parameter$whichlevel==1){
-					names(df)[names(df)=="stratum"]="state"
-					names(df)[names(df)=="tostratum"]="tostate"
+				df=expand.grid(stratum=sl[1:nstrata],occ=occasions,id=factor(1:nrow(x$data)))
+				df$seq=1:nrow(df)
+			}
+			# two-level stratification
+			if(!is.null(x$strata.list))
+			{
+				state.df=data.frame(stratum=sl[1:nstrata],state=states,oth=oth)	
+				df=merge(df,state.df,by="stratum")
+				if(parameter$tostrata)
+				{
+					state.df=data.frame(tostratum=sl[1:nstrata],tostate=states,tooth=oth)	
+					df=merge(df,state.df,by="tostratum")
 				} 
-				if(parameter$whichlevel==2) {
-					names(df)[names(df)=="stratum"]=oth.name
-					names(df)[names(df)=="tostratum"]=paste("to",oth.name,sep="")
-				}	 
+				if(parameter$whichlevel!=0)
+				{
+					if(parameter$whichlevel==1){
+						names(df)[names(df)=="stratum"]="state"
+						names(df)[names(df)=="tostratum"]="tostate"
+					} 
+					if(parameter$whichlevel==2) {
+						names(df)[names(df)=="stratum"]=oth.name
+						names(df)[names(df)=="tostratum"]=paste("to",oth.name,sep="")
+					}	 
+				}else
+				{
+					names(df)[names(df)=="oth"]=oth.name
+					names(df)[names(df)=="tooth"]=paste("to",oth.name,sep="")
+				}
 			}else
 			{
-				names(df)[names(df)=="oth"]=oth.name
-				names(df)[names(df)=="tooth"]=paste("to",oth.name,sep="")
+				if(parameter$whichlevel!=0)
+				{
+					if(parameter$whichlevel==1)names(df)[names(df)=="stratum"]="state"
+					if(parameter$whichlevel==2) names(df)[names(df)=="stratum"]=oth.name
+					if(parameter$whichlevel==1)names(df)[names(df)=="tostratum"]="tostate"
+					if(parameter$whichlevel==2) names(df)[names(df)=="tostratum"]=paste("to",oth.name,sep="")
+				}	 
 			}
-	    }else
-	    {
-			if(parameter$whichlevel!=0)
-			{
-				if(parameter$whichlevel==1)names(df)[names(df)=="stratum"]="state"
-				if(parameter$whichlevel==2) names(df)[names(df)=="stratum"]=oth.name
-				if(parameter$whichlevel==1)names(df)[names(df)=="tostratum"]="tostate"
-				if(parameter$whichlevel==2) names(df)[names(df)=="tostratum"]=paste("to",oth.name,sep="")
-			}	 
-  	    }
+		}
 	}
 	return(df)
 }
