@@ -183,6 +183,7 @@
 #' @param extra.args optional character string that is passed to admb if use.admb==TRUE
 #' @param strata.labels labels for strata used in capture history; they are converted to numeric in the order listed. Only needed to specify unobserved strata. For any unobserved strata p=0..
 #' @param clean if TRUE, deletes the tpl and executable files for amdb if use.admb=T
+#' @param save.matrices for HMM models this option controls whether the gamma,dmat and delta matrices are saved in the model object
 #' @param ... optional arguments passed to js or cjs and optimx
 #' @return crm model object with class=("crm",submodel) where submodel is
 #' either "CJS" or "JS" at present.
@@ -194,7 +195,7 @@
 #' \code{\link{make.design.data}},\code{\link{process.data}}
 #' @keywords models
 #' @examples
-#' 
+#' {
 #' # cormack-jolly-seber model
 #' # fit 3 cjs models with crm
 #' data(dipper)
@@ -203,52 +204,53 @@
 #' mod.Phit.pt=crm(dipper.proc,dipper.ddl,
 #'    model.parameters=list(Phi=list(formula=~time),p=list(formula=~time)))
 #' mod.Phit.pt
-#' mod.Phidot.pdot=crm(dipper.proc,dipper.ddl,
-#'    model.parameters=list(Phi=list(formula=~1),p=list(formula=~1)))
-#' mod.Phidot.pdot
 #' mod.Phisex.pdot=crm(dipper.proc,dipper.ddl,groups="sex",
 #'    model.parameters=list(Phi=list(formula=~sex),p=list(formula=~1)))
 #' mod.Phisex.pdot
-#' # fit same 3 models with calls to mark
-#' \donttest{
-#' require(RMark)
-#' mod0=mark(dipper,
-#'    model.parameters=list(Phi=list(formula=~time),p=list(formula=~time)),output=FALSE)
-#' summary(mod0,brief=TRUE)
-#' mod1=mark(dipper,
-#'    model.parameters=list(Phi=list(formula=~1),p=list(formula=~1)),output=FALSE)
-#' summary(mod1,brief=TRUE)
-#' mod2<-mark(dipper,groups="sex",
-#'    model.parameters=list(Phi=list(formula=~sex),p=list(formula=~1)),output=FALSE)
-#' summary(mod2,brief=TRUE)
-#' }
+#' ## if you have RMark installed you can use this code to run the same models 
+#' ## by removing the comment symbol
+#' #library(RMark)
+#' #data(dipper)
+#' #mod0=mark(dipper,
+#' #model.parameters=list(Phi=list(formula=~time),p=list(formula=~time)),output=FALSE)
+#' #summary(mod0,brief=TRUE)
+#' #mod1=mark(dipper,
+#' #model.parameters=list(Phi=list(formula=~1),p=list(formula=~1)),output=FALSE)
+#' #summary(mod1,brief=TRUE)
+#' #mod2<-mark(dipper,groups="sex",
+#' #model.parameters=list(Phi=list(formula=~sex),p=list(formula=~1)),output=FALSE)
+#' #summary(mod2,brief=TRUE)
 #' # jolly seber model
 #' crm(dipper,model="js",groups="sex",
 #'    model.parameters=list(pent=list(formula=~sex),N=list(formula=~sex)),accumulate=FALSE)
 #' \donttest{
-#' mark(dipper,model="POPAN",groups="sex",
-#'    model.parameters=list(pent=list(formula=~sex),N=list(formula=~sex)))
-#' library(RMark)
-#' data(dipper)
-#' data(mstrata)
-#' mark(dipper,model.parameters=list(p=list(formula=~time)),output=FALSE)$results$beta
-#' mark(mstrata,model="Multistrata",model.parameters=list(p=list(formula=~1),
-#'  S=list(formula=~1),Psi=list(formula=~-1+stratum:tostratum)),
-#'  output=FALSE)$results$beta
-#' detach("package:RMark")
-#' ##CJS example
+#' # This example is excluded from testing to reduce package check time
+#' # if you have RMark installed you can use this code to run the same models 
+#' # by removing the comment 
+#' #data(dipper)
+#' #data(mstrata)
+#' #mark(dipper,model.parameters=list(p=list(formula=~time)),output=FALSE)$results$beta
+#' #mark(mstrata,model="Multistrata",model.parameters=list(p=list(formula=~1),
+#' # S=list(formula=~1),Psi=list(formula=~-1+stratum:tostratum)),
+#' # output=FALSE)$results$beta
+#' #mod=mark(dipper,model="POPAN",groups="sex",
+#' #   model.parameters=list(pent=list(formula=~sex),N=list(formula=~sex)))
+#' #summary(mod)
+#' #CJS example with hmm
 #' crm(dipper,model="hmmCJS",model.parameters = list(p = list(formula = ~time)))
-#' ##MSCJS example
-#' ms=process.data(mstrata,model="hmmMSCJS")
+#' ##MSCJS example with hmm
+#' data(mstrata)
+#' ms=process.data(mstrata,model="hmmMSCJS",strata.labels=c("A","B","C"))
 #' ms.ddl=make.design.data(ms)
 #' ms.ddl$Psi$fix=NA
 #' ms.ddl$Psi$fix[ms.ddl$Psi$stratum==ms.ddl$Psi$tostratum]=1
 #' crm(ms,ms.ddl,model.parameters=list(Psi=list(formula=~-1+stratum:tostratum)))
 #' }
+#' }
 crm <- function(data,ddl=NULL,begin.time=1,model="CJS",title="",model.parameters=list(),design.parameters=list(),initial=NULL,
  groups = NULL, time.intervals = NULL,debug=FALSE, method="BFGS", hessian=FALSE, accumulate=TRUE,chunk_size=1e7, 
- control=NULL,refit=1,itnmax=5000,scale=NULL,run=TRUE,burnin=100,iter=1000,use.admb=FALSE,crossed=NULL,reml=FALSE,compile=FALSE,extra.args=NULL,
- strata.labels=NULL,clean=TRUE,...)
+ control=list(),refit=1,itnmax=5000,scale=NULL,run=TRUE,burnin=100,iter=1000,use.admb=FALSE,crossed=NULL,reml=FALSE,compile=FALSE,extra.args=NULL,
+ strata.labels=NULL,clean=TRUE,save.matrices=TRUE,...)
 {
 model=toupper(model)
 ptm=proc.time()
@@ -310,13 +312,24 @@ if(is.null(ddl))
 	message("Creating design data...\n")
 	flush.console()
 	ddl=make.design.data(data.proc,design.parameters)
-	ddl=set.fixed(ddl,parameters) #   setup fixed values if old way used
 } else
+{
+	for (i in 1:length(parameters))
+	{
+		if(!is.null(ddl[[i]]$order))
+		   if(any(ddl[[i]]$order!=1:nrow(ddl[[i]]))) 
+			   stop(paste("Design data for parameter",names(parameters)[i],"is out of order."))
+	}
+    if(!is.null(design.parameters))
+		for(parname in names(design.parameters))
+			ddl$design.parameters[[parname]]=c(ddl$design.parameters[[parname]],design.parameters[[parname]])
 	design.parameters=ddl$design.parameters
+}
+ddl=set.fixed(ddl,parameters) #   setup fixed values if old way used
 # Create design matrices for each parameter
 dml=create.dml(ddl,model.parameters=parameters,design.parameters=design.parameters,chunk_size=1e7)
 # For HMM call set.initial to get ptype and set initial values
-if(substr(model,1,3)=="HMM")
+if(substr(model,1,3)=="HMM"|(nchar(model)>=4 &substr(model,1,4)=="MVMS"))
 	initial.list=set.initial(names(dml),dml,initial)
 # if not running, return object with data,ddl,dml etc
 if(!run) return(list(model=model,data=data.proc,model.parameters=parameters,design.parameters=design.parameters,ddl=ddl,dml=dml,results=initial.list))
@@ -328,7 +341,11 @@ if("SANN"%in%method)
 	method="SANN"
     control$maxit=itnmax
 }
-if("nlminb"%in%method)control$eval.max=itnmax
+if("nlminb"%in%method)
+{
+	control$eval.max=itnmax
+	control$iter.max=itnmax
+}
 # Call estimation function which depends on the model
 message("Fitting model\n")
 if(model=="CJS")
@@ -351,36 +368,43 @@ if(model=="PROBITCJS")
 	    runmodel=probitCJS(ddl,dml,parameters=parameters,design.parameters=design.parameters,
 					   initial=initial,iter=iter,burnin=burnin)	   
 }
-if(substr(model,1,3)=="HMM")
+if(substr(model,1,3)=="HMM"|(nchar(model)>=4 &substr(model,1,4)=="MVMS"))
 {
-	if(is.null(data.proc$strata.list))
+	if(substr(model,1,4)=="MVMS")
 	{
-	   runmodel=optim(unlist(initial.list$par),HMMLikelihood,type=initial.list$ptype,x=data.proc$ehmat,m=data.proc$m,T=data.proc$nocc,start=data.proc$start,freq=data.proc$freq,
-				fct_dmat=data.proc$fct_dmat,fct_gamma=data.proc$fct_gamma,fct_delta=data.proc$fct_delta,ddl=ddl,dml=dml,parameters=parameters,control=control,
-				method=method,debug=debug,hessian=hessian)
-	   runmodel$mat=HMMLikelihood(par=runmodel$par,type=initial.list$ptype,x=data.proc$ehmat,m=data.proc$m,T=data.proc$nocc,start=data.proc$start,freq=data.proc$freq,
-			fct_dmat=data.proc$fct_dmat,fct_gamma=data.proc$fct_gamma,fct_delta=data.proc$fct_delta,ddl=ddl,dml=dml,parameters=parameters,return.mat=TRUE)
-    } else
-	{
-		m=list(ns=length(data.proc$strata.list$states),na=length(data.proc$strata.list[[names(data.proc$strata.list)[names(data.proc$strata.list)!="states"]]]))
-		runmodel=optim(unlist(initial.list$par),HMMLikelihood,type=initial.list$ptype,x=data.proc$ehmat,m=m,T=data.proc$nocc,start=data.proc$start,freq=data.proc$freq,
-				fct_dmat=data.proc$fct_dmat,fct_gamma=data.proc$fct_gamma,fct_delta=data.proc$fct_delta,ddl=ddl,dml=dml,parameters=parameters,control=control,
-				method=method,debug=debug,hessian=hessian)
-#		runmodel$mat=HMMLikelihood(par=runmodel$par,type=initial.list$ptype,x=data.proc$ehmat,m=m,T=data.proc$nocc,start=data.proc$start,freq=data.proc$freq,
-#				fct_dmat=data.proc$fct_dmat,fct_gamma=data.proc$fct_gamma,fct_delta=data.proc$fct_delta,ddl=ddl,dml=dml,parameters=parameters,return.mat=TRUE)
+		obslevels=data.proc$ObsLevels
+		sup=data.proc$fct_sup(list(obslevels=obslevels))
+	} else
+		sup=NULL
+	if(is.null(data.proc$strata.list) | substr(model,1,4)=="MVMS"){
+		mx=data.proc$m
+	}else{
+		mx=list(ns=length(data.proc$strata.list$states),na=length(data.proc$strata.list[[names(data.proc$strata.list)[names(data.proc$strata.list)!="states"]]]))
 	}
+	runmodel=optimx(unlist(initial.list$par),HMMLikelihood,method=method,debug=debug,hessian=hessian,itnmax=itnmax,xx=data.proc$ehmat,mx=mx,
+			        type=initial.list$ptype,T=data.proc$nocc,xstart=data.proc$start,freq=data.proc$freq,control=control,
+				    fct_dmat=data.proc$fct_dmat,fct_gamma=data.proc$fct_gamma,fct_delta=data.proc$fct_delta,ddl=ddl,dml=dml,
+					parameters=parameters,sup=sup)
+	par <- coef(runmodel, order="value")[1, ]
+	runmodel=list(optim.details=as.list(summary(runmodel, order="value",par.select=FALSE)[1, ]))
+	if(hessian)runmodel$hessian=attr(runmodel$optim.details,"details")$nhatend
+	runmodel$convergence=runmodel$optim.details$convcode
+	runmodel$options=list(accumulate=accumulate,initial=initial.list$par,method=method,
+	                		chunk_size=chunk_size,itnmax=itnmax,control=control)
+ 	if(save.matrices)
+		runmodel$mat=HMMLikelihood(par=par,type=initial.list$ptype,xx=data.proc$ehmat,mx=mx,T=data.proc$nocc,xstart=data.proc$start,freq=data.proc$freq,
+			fct_dmat=data.proc$fct_dmat,fct_gamma=data.proc$fct_gamma,fct_delta=data.proc$fct_delta,ddl=ddl,dml=dml,parameters=parameters,return.mat=TRUE,sup=sup)
+	parlist=split(par,initial.list$ptype)
 	par=vector("list",length=length(names(initial.list$par)))
 	names(par)=names(initial.list$par)
-	#par=split(runmodel$par,initial.list$ptype)
-	for(p in names(initial.list$par))
+	for(p in names(parlist))
 	{
-		par[[p]]=runmodel$par[initial.list$ptype==p]
+		par[[p]]=parlist[[p]]
 		names(par[[p]])=colnames(dml[[p]]$fe)	
 	}
 	runmodel$beta=par
 	runmodel$par=NULL
-	runmodel$neg2lnl=2*runmodel$value
-	runmodel$value=NULL
+	runmodel$neg2lnl=2*runmodel$optim.details$value
 	runmodel$AIC=runmodel$neg2lnl+2*sum(sapply(runmodel$beta,length))
 	if(!is.null(runmodel$hessian))
 	{
@@ -389,22 +413,20 @@ if(substr(model,1,3)=="HMM")
 		rownames(runmodel$beta.vcv)=colnames(runmodel$beta.vcv)
 	}
 	class(runmodel)=c("crm","mle",model)
-#	runmodel$model_data=list(ddl=ddl)
 }
 #
 # Return fitted MARK model object or if external, return character string with same class and save file
 if(!is.null(runmodel$convergence) && runmodel$convergence!=0&!use.admb)
 {
 	warning("******Model did not converge******")
-	msg=attr(runmodel$optim.details,"details")[[1]]$message
+	msg=attr(runmodel$optim.details,"details")$message
 	if(is.null(msg)) msg="Exceeded maximum number of iterations"
 	warning(msg)
 }
 
 object=list(model=model,data=data.proc,model.parameters=parameters,design.parameters=design.parameters,results=runmodel)
 class(object)=class(runmodel)
-#if(!re & model!="MSCJS"& toupper(substr(model,1,3))!="HMM")
-if(!re & model!="MSCJS")
+if(!re & model!="MSCJS" & (nchar(model)>=4 & substr(model,1,4)!="MVMS"))
    object$results$reals=predict(object,ddl=ddl,unique=TRUE,se=hessian)
 cat(paste("\nElapsed time in minutes: ",round((proc.time()[3]-ptm[3])/60,digits=4),"\n"))
 return(object)
